@@ -1,9 +1,9 @@
-import asyncio
 import inspect
 import logging
 from collections.abc import Callable
-from typing import Any, Literal, Optional
+from typing import Any, Awaitable, Literal, Optional
 
+from fastapi.exceptions import HTTPException
 from fastapi.security import OAuth2AuthorizationCodeBearer, SecurityScopes
 from fastapi.security.base import SecurityBase
 from jose import jwt
@@ -21,11 +21,12 @@ class AzureAuthorizationCodeBearerBase(SecurityBase):
     def __init__(
         self,
         app_client_id: str,
+        auto_error: bool = True,
         tenant_id: Optional[str] = None,
         scopes: Optional[dict[str, str]] = None,
         multi_tenant: bool = False,
         validate_iss: bool = True,
-        iss_callable: Optional[Callable[..., Any]] = None,
+        iss_callable: Optional[Callable[[str], Awaitable[str]]] = None,
         token_version: Literal[1, 2] = 2,
         openid_config_use_app_id: bool = False,
         openapi_authorization_url: Optional[str] = None,
@@ -37,6 +38,8 @@ class AzureAuthorizationCodeBearerBase(SecurityBase):
 
         :param app_client_id: str
             Your applications client ID. This will be the `Web app` in Azure AD
+        :param auto_error: bool
+            Whether to throw exceptions or return None on __call__.
         :param tenant_id: str
             Your Azure tenant ID, only needed for single tenant apps
         :param scopes: Optional[dict[str, str]
@@ -51,7 +54,7 @@ class AzureAuthorizationCodeBearerBase(SecurityBase):
         :param validate_iss: bool
         **Only used for multi-tenant applications**
             Whether to validate the token `iss` (issuer) or not. This can be skipped to allow anyone to log in.
-        :param iss_callable: Callable
+        :param iss_callable: Async Callable
         **Only used for multi-tenant application**
             Async function that has to accept a `tid` (tenant ID) and return a `iss` (issuer) or
              raise an InvalidIssuer exception
@@ -72,12 +75,11 @@ class AzureAuthorizationCodeBearerBase(SecurityBase):
             Override OpenAPI description
 
         """
+        self.auto_error = auto_error
         # Validate settings, making sure there's no misconfigured dependencies out there
         if multi_tenant:
             if validate_iss and not callable(iss_callable):
                 raise RuntimeError('`validate_iss` is enabled, so you must provide an `iss_callable`')
-            elif iss_callable and not asyncio.iscoroutinefunction(iss_callable):
-                raise RuntimeError('`iss_callable` must be a coroutine')
             elif iss_callable and 'tid' not in inspect.signature(iss_callable).parameters.keys():
                 raise RuntimeError('`iss_callable` must accept `tid` as an argument')
 
@@ -115,92 +117,97 @@ class AzureAuthorizationCodeBearerBase(SecurityBase):
             scopes=scopes,
             scheme_name='AzureAuthorizationCodeBearerBase',
             description=openapi_description or '`Leave client_secret blank`',
-            auto_error=True,
+            auto_error=True,  # We catch this exception in __call__
         )
         self.model = self.oauth.model
 
-    async def __call__(self, request: Request, security_scopes: SecurityScopes) -> User:
+    async def __call__(self, request: Request, security_scopes: SecurityScopes) -> Optional[User]:
         """
         Extends call to also validate the token.
         """
-        access_token = await self.oauth(request=request)
         try:
-            # Extract header information of the token.
-            header: dict[str, str] = jwt.get_unverified_header(token=access_token) or {}
-            claims: dict[str, Any] = jwt.get_unverified_claims(token=access_token) or {}
-        except Exception as error:
-            log.warning('Malformed token received. %s. Error: %s', access_token, error, exc_info=True)
-            raise InvalidAuth(detail='Invalid token format')
+            access_token = await self.oauth(request=request)
+            try:
+                # Extract header information of the token.
+                header: dict[str, str] = jwt.get_unverified_header(token=access_token) or {}
+                claims: dict[str, Any] = jwt.get_unverified_claims(token=access_token) or {}
+            except Exception as error:
+                log.warning('Malformed token received. %s. Error: %s', access_token, error, exc_info=True)
+                raise InvalidAuth(detail='Invalid token format')
 
-        for scope in security_scopes.scopes:
-            token_scope_string = claims.get('scp', '')
-            if isinstance(token_scope_string, str):
-                token_scopes = token_scope_string.split(' ')
-                if scope not in token_scopes:
-                    raise InvalidAuth('Required scope missing')
+            for scope in security_scopes.scopes:
+                token_scope_string = claims.get('scp', '')
+                if isinstance(token_scope_string, str):
+                    token_scopes = token_scope_string.split(' ')
+                    if scope not in token_scopes:
+                        raise InvalidAuth('Required scope missing')
+                else:
+                    raise InvalidAuth('Token contains invalid formatted scopes')
+
+            # Load new config if old
+            await self.openid_config.load_config()
+
+            if self.multi_tenant and self.validate_iss and self.iss_callable:
+                iss = await self.iss_callable(tid=claims.get('tid'))
             else:
-                raise InvalidAuth('Token contains invalid formatted scopes')
+                iss = self.openid_config.issuer
 
-        # Load new config if old
-        await self.openid_config.load_config()
-
-        if self.multi_tenant and self.validate_iss and self.iss_callable:
-            iss = await self.iss_callable(tid=claims.get('tid'))
-        else:
-            iss = self.openid_config.issuer
-
-        # Use the `kid` from the header to find a matching signing key to use
-        try:
-            if key := self.openid_config.signing_keys.get(header.get('kid', '')):
-                # We require and validate all fields in an Azure AD token
-                options = {
-                    'verify_signature': True,
-                    'verify_aud': True,
-                    'verify_iat': True,
-                    'verify_exp': True,
-                    'verify_nbf': True,
-                    'verify_iss': self.validate_iss,
-                    'verify_sub': True,
-                    'verify_jti': True,
-                    'verify_at_hash': True,
-                    'require_aud': True,
-                    'require_iat': True,
-                    'require_exp': True,
-                    'require_nbf': True,
-                    'require_iss': self.validate_iss,
-                    'require_sub': True,
-                    'require_jti': False,
-                    'require_at_hash': False,
-                    'leeway': 0,
-                }
-                # Validate token
-                token = jwt.decode(
-                    access_token,
-                    key=key,
-                    algorithms=['RS256'],
-                    audience=self.app_client_id if self.token_version == 2 else f'api://{self.app_client_id}',
-                    issuer=iss,
-                    options=options,
-                )
-                # Attach the user to the request. Can be accessed through `request.state.user`
-                user: User = User(**token | {'claims': token, 'access_token': access_token})
-                request.state.user = user
-                return user
-        except JWTClaimsError as error:
-            log.info('Token contains invalid claims. %s', error)
-            raise InvalidAuth(detail='Token contains invalid claims')
-        except ExpiredSignatureError as error:
-            log.info('Token signature has expired. %s', error)
-            raise InvalidAuth(detail='Token signature has expired')
-        except JWTError as error:
-            log.warning('Invalid token. Error: %s', error, exc_info=True)
-            raise InvalidAuth(detail='Unable to validate token')
-        except Exception as error:
-            # Extra failsafe in case of a bug in a future version of the jwt library
-            log.exception('Unable to process jwt token. Uncaught error: %s', error)
-            raise InvalidAuth(detail='Unable to process token')
-        log.warning('Unable to verify token. No signing keys found')
-        raise InvalidAuth(detail='Unable to verify token, no signing keys found')
+            # Use the `kid` from the header to find a matching signing key to use
+            try:
+                if key := self.openid_config.signing_keys.get(header.get('kid', '')):
+                    # We require and validate all fields in an Azure AD token
+                    options = {
+                        'verify_signature': True,
+                        'verify_aud': True,
+                        'verify_iat': True,
+                        'verify_exp': True,
+                        'verify_nbf': True,
+                        'verify_iss': self.validate_iss,
+                        'verify_sub': True,
+                        'verify_jti': True,
+                        'verify_at_hash': True,
+                        'require_aud': True,
+                        'require_iat': True,
+                        'require_exp': True,
+                        'require_nbf': True,
+                        'require_iss': self.validate_iss,
+                        'require_sub': True,
+                        'require_jti': False,
+                        'require_at_hash': False,
+                        'leeway': 0,
+                    }
+                    # Validate token
+                    token = jwt.decode(
+                        access_token,
+                        key=key,
+                        algorithms=['RS256'],
+                        audience=self.app_client_id if self.token_version == 2 else f'api://{self.app_client_id}',
+                        issuer=iss,
+                        options=options,
+                    )
+                    # Attach the user to the request. Can be accessed through `request.state.user`
+                    user: User = User(**token | {'claims': token, 'access_token': access_token})
+                    request.state.user = user
+                    return user
+            except JWTClaimsError as error:
+                log.info('Token contains invalid claims. %s', error)
+                raise InvalidAuth(detail='Token contains invalid claims')
+            except ExpiredSignatureError as error:
+                log.info('Token signature has expired. %s', error)
+                raise InvalidAuth(detail='Token signature has expired')
+            except JWTError as error:
+                log.warning('Invalid token. Error: %s', error, exc_info=True)
+                raise InvalidAuth(detail='Unable to validate token')
+            except Exception as error:
+                # Extra failsafe in case of a bug in a future version of the jwt library
+                log.exception('Unable to process jwt token. Uncaught error: %s', error)
+                raise InvalidAuth(detail='Unable to process token')
+            log.warning('Unable to verify token. No signing keys found')
+            raise InvalidAuth(detail='Unable to verify token, no signing keys found')
+        except (HTTPException, InvalidAuth):
+            if not self.auto_error:
+                return None
+            raise
 
 
 class SingleTenantAzureAuthorizationCodeBearer(AzureAuthorizationCodeBearerBase):
@@ -208,6 +215,7 @@ class SingleTenantAzureAuthorizationCodeBearer(AzureAuthorizationCodeBearerBase)
         self,
         app_client_id: str,
         tenant_id: str,
+        auto_error: bool = True,
         scopes: Optional[dict[str, str]] = None,
         token_version: Literal[1, 2] = 2,
         openid_config_use_app_id: bool = False,
@@ -222,6 +230,8 @@ class SingleTenantAzureAuthorizationCodeBearer(AzureAuthorizationCodeBearerBase)
             Your applications client ID. This will be the `Web app` in Azure AD
         :param tenant_id: str
             Your Azure tenant ID, only needed for single tenant apps
+        :param auto_error: bool
+            Whether to throw exceptions or return None on __call__.
         :param scopes: Optional[dict[str, str]
             Scopes, these are the ones you've configured in Azure AD. Key is scope, value is a description.
             Example:
@@ -245,6 +255,7 @@ class SingleTenantAzureAuthorizationCodeBearer(AzureAuthorizationCodeBearerBase)
         """
         super().__init__(
             app_client_id=app_client_id,
+            auto_error=auto_error,
             tenant_id=tenant_id,
             scopes=scopes,
             token_version=token_version,
@@ -260,9 +271,10 @@ class MultiTenantAzureAuthorizationCodeBearer(AzureAuthorizationCodeBearerBase):
     def __init__(
         self,
         app_client_id: str,
+        auto_error: bool = True,
         scopes: Optional[dict[str, str]] = None,
         validate_iss: bool = True,
-        iss_callable: Optional[Callable[..., Any]] = None,
+        iss_callable: Optional[Callable[[str], Awaitable[str]]] = None,
         openid_config_use_app_id: bool = False,
         openapi_authorization_url: Optional[str] = None,
         openapi_token_url: Optional[str] = None,
@@ -273,6 +285,8 @@ class MultiTenantAzureAuthorizationCodeBearer(AzureAuthorizationCodeBearerBase):
 
         :param app_client_id: str
             Your applications client ID. This will be the `Web app` in Azure AD
+        :param auto_error: bool
+            Whether to throw exceptions or return None on __call__.
         :param scopes: Optional[dict[str, str]
             Scopes, these are the ones you've configured in Azure AD. Key is scope, value is a description.
             Example:
@@ -282,7 +296,7 @@ class MultiTenantAzureAuthorizationCodeBearer(AzureAuthorizationCodeBearerBase):
 
         :param validate_iss: bool
             Whether to validate the token `iss` (issuer) or not. This can be skipped to allow anyone to log in.
-        :param iss_callable: Callable
+        :param iss_callable: Async Callable
             Async function that has to accept a `tid` (tenant ID) and return a `iss` (issuer) or
              raise an InvalidIssuer exception
             This is required when validate_iss is set to `True`.
@@ -300,6 +314,7 @@ class MultiTenantAzureAuthorizationCodeBearer(AzureAuthorizationCodeBearerBase):
         """
         super().__init__(
             app_client_id=app_client_id,
+            auto_error=auto_error,
             scopes=scopes,
             validate_iss=validate_iss,
             iss_callable=iss_callable,
